from __future__ import annotations

from logging import Logger, getLogger
from typing import Type

from kafka.coordinator.assignors.abstract import AbstractPartitionAssignor
from kafka.coordinator.assignors.roundrobin import RoundRobinPartitionAssignor

from buz.kafka.domain.models.consumer_initial_offset_position import ConsumerInitialOffsetPosition
from buz.kafka.domain.models.kafka_connection_config import KafkaConnectionConfig
from buz.kafka.infrastructure.deserializers.byte_deserializer import ByteDeserializer
from buz.kafka.infrastructure.deserializers.implementations.json_byte_deserializer import JSONByteDeserializer
from buz.kafka.infrastructure.kafka_python.kafka_python_multi_threaded_consumer import KafkaPythonMultiThreadedConsumer
from buz.kafka.infrastructure.serializers.kafka_header_serializer import KafkaHeaderSerializer


class KafkaPythonMultiThreadedConsumerFactory:
    __FALLBACK_PARTITION_ASSIGNORS = (RoundRobinPartitionAssignor,)

    def __init__(
        self,
        consumer_group: str,
        topics: list[str],
        kafka_connection_config: KafkaConnectionConfig,
        initial_offset_position: ConsumerInitialOffsetPosition = ConsumerInitialOffsetPosition.BEGINNING,
        byte_deserializer: ByteDeserializer = JSONByteDeserializer(),
        header_serializer: KafkaHeaderSerializer = KafkaHeaderSerializer(),
        kafka_partition_assignors: tuple[Type[AbstractPartitionAssignor], ...] = (),
        logger: Logger = getLogger(),
    ) -> None:
        self.__consumer_group = consumer_group
        self.__topics = topics
        self.__kafka_connection_config = kafka_connection_config
        self.__initial_offset_position = initial_offset_position
        self.__byte_deserializer = byte_deserializer
        self.__header_serializer = header_serializer
        self.__kafka_partition_assignors = self.__get_partition_assignors_ordered_by_priority(kafka_partition_assignors)
        self.__logger = logger

    def __get_partition_assignors_ordered_by_priority(
        self, kafka_partition_assignors: tuple[Type[AbstractPartitionAssignor], ...]
    ) -> tuple[Type[AbstractPartitionAssignor], ...]:
        # A tuple is used to support rolling-updates.
        return tuple(kafka_partition_assignors + self.__FALLBACK_PARTITION_ASSIGNORS)

    def build(self) -> KafkaPythonMultiThreadedConsumer:
        return KafkaPythonMultiThreadedConsumer(
            consumer_group=self.__consumer_group,
            topics=self.__topics,
            connection_config=self.__kafka_connection_config,
            initial_offset_position=self.__initial_offset_position,
            byte_deserializer=self.__byte_deserializer,
            header_serializer=self.__header_serializer,
            partition_assignors=self.__kafka_partition_assignors,
            logger=self.__logger,
        )



from abc import abstractmethod, ABC
from typing import Callable

from buz.kafka.domain.models.kafka_consumer_record import KafkaConsumerRecord

DEFAULT_NUMBER_OF_MESSAGES_TO_POLL = 500


class KafkaConsumer(ABC):
    @abstractmethod
    def consume(
        self,
        *,
        consumption_callback: Callable[[KafkaConsumerRecord], None],
        number_of_messages_to_poll: int = DEFAULT_NUMBER_OF_MESSAGES_TO_POLL,
    ) -> None:
        pass

    def request_stop(self) -> None:
        """Request a graceful stop.
        This method does not stop the consumer in an instantaneous way,
        it will eventually finalize when the current task is completed
        """
        pass



import threading
from concurrent.futures import ThreadPoolExecutor
from datetime import timedelta, datetime
from logging import Logger
from time import sleep
from typing import Optional, Type, TypeVar

from kafka.coordinator.assignors.abstract import AbstractPartitionAssignor

from buz.event import Event, Subscriber
from buz.event.consumer import Consumer
from buz.event.domain.queue.queue_repository import QueueRepository
from buz.event.infrastructure.buz_kafka.consume_strategy.consume_strategy import KafkaConsumeStrategy
from buz.event.infrastructure.buz_kafka.consume_strategy.kafka_on_fail_strategy import KafkaOnFailStrategy
from buz.event.middleware.consume_middleware import ConsumeMiddleware
from buz.event.middleware.consume_middleware_chain_resolver import ConsumeMiddlewareChainResolver
from buz.event.strategies.retry.consume_retrier import ConsumeRetrier
from buz.event.strategies.retry.reject_callback import RejectCallback
from buz.kafka import (
    KafkaConnectionConfig,
    KafkaConsumerRecord,
    ConsumerInitialOffsetPosition,
)
from buz.kafka.infrastructure.deserializers.bytes_to_message_deserializer import BytesToMessageDeserializer
from buz.kafka.infrastructure.deserializers.implementations.json_bytes_to_message_deserializer import (
    JSONBytesToMessageDeserializer,
)
from buz.kafka.infrastructure.kafka_python.factories.kafka_python_multi_threaded_consumer_factory import (
    KafkaPythonMultiThreadedConsumerFactory,
)
from buz.kafka.infrastructure.kafka_python.kafka_poll_record import KafkaPollRecord
from buz.kafka.infrastructure.kafka_python.kafka_python_multi_threaded_consumer import KafkaPythonMultiThreadedConsumer

T = TypeVar("T", bound=Event)
KafkaConsumer = KafkaPythonMultiThreadedConsumer  # TODO: remove this alias once we comply with the interface


class BuzKafkaMultiThreadedConsumer(Consumer):
    def __init__(
        self,
        *,
        connection_config: KafkaConnectionConfig,
        consume_strategy: KafkaConsumeStrategy,
        on_fail_strategy: KafkaOnFailStrategy,
        queue_repository: QueueRepository[tuple[KafkaConsumer, KafkaPollRecord]],
        kafka_partition_assignors: tuple[Type[AbstractPartitionAssignor], ...] = (),
        subscribers: list[Subscriber],
        logger: Logger,
        consumer_initial_offset_position: ConsumerInitialOffsetPosition,
        deserializers_per_subscriber: dict[Subscriber, BytesToMessageDeserializer[T]],
        consume_middlewares: Optional[list[ConsumeMiddleware]] = None,
        consume_retrier: Optional[ConsumeRetrier] = None,
        reject_callback: Optional[RejectCallback] = None,
        max_queue_size: int,
        max_records_retrieved_per_poll: int,
        min_time_between_polls_in_ms: int,
    ):
        self.__connection_config = connection_config
        self.__consume_strategy = consume_strategy
        self.__on_fail_strategy = on_fail_strategy
        self.__queue_repository = queue_repository
        self.__kafka_partition_assignors = kafka_partition_assignors
        self.__subscribers = subscribers
        self.__logger = logger
        self.__consumer_initial_offset_position = consumer_initial_offset_position
        self.__deserializers_per_subscriber = deserializers_per_subscriber
        self.__consume_middleware_chain_resolver = ConsumeMiddlewareChainResolver(consume_middlewares or [])
        self.__consume_retrier = consume_retrier
        self.__reject_callback = reject_callback
        self.__max_queue_size = max_queue_size
        self.__max_records_retrieved_per_poll = max_records_retrieved_per_poll
        self.__min_time_between_polls = timedelta(milliseconds=min_time_between_polls_in_ms)

        self.__subscriber_per_consumer_mapper: dict[KafkaConsumer, Subscriber] = {}
        self.__polling_threads_pool: ThreadPoolExecutor = ThreadPoolExecutor()
        self.__kafka_consumers_with_poll_requested: set[KafkaConsumer] = set()
        self.__should_stop = threading.Event()

    def run(self) -> None:
        self.__start_all_kafka_consumers_concurrently()

        for kafka_consumer, subscriber in self.__subscriber_per_consumer_mapper.items():
            self.__request_more_records_asynchronously(kafka_consumer=kafka_consumer, last_time_polled=datetime.now())

        while not self.__should_stop.is_set():
            if self.__queue_repository.is_empty():
                sleep(0.1)
                continue
            self.__consume_kafka_poll_records_from_queue()

        self.__perform_graceful_stop()

    def __start_all_kafka_consumers_concurrently(self):
        startup_threads_pool = ThreadPoolExecutor()
        for subscriber in self.__subscribers:
            startup_threads_pool.submit(self.__create_kafka_consumer_for_subscriber, subscriber)

        startup_threads_pool.shutdown()

    def __create_kafka_consumer_for_subscriber(self, subscriber: Subscriber) -> None:
        byte_deserializer = self.__deserializers_per_subscriber.get(subscriber)
        kafka_python_consumer_factory = KafkaPythonMultiThreadedConsumerFactory(
            consumer_group=self.__consume_strategy.get_subscription_group(subscriber),
            topics=self.__consume_strategy.get_topics(subscriber),
            kafka_connection_config=self.__connection_config,
            initial_offset_position=self.__consumer_initial_offset_position,
            byte_deserializer=byte_deserializer or JSONBytesToMessageDeserializer(event_class=subscriber.handles()),  # type: ignore[arg-type]
            kafka_partition_assignors=self.__kafka_partition_assignors,
            logger=self.__logger,
        )
        kafka_consumer = kafka_python_consumer_factory.build()
        self.__subscriber_per_consumer_mapper[kafka_consumer] = subscriber

    def __poll_kafka_consumer(self, kafka_consumer: KafkaConsumer, last_time_polled: datetime) -> None:
        self.__kafka_consumers_with_poll_requested.remove(kafka_consumer)

        if self.__should_stop.is_set():
            return

        queue_size = self.__queue_repository.get_size()
        available_space_in_queue = self.__max_queue_size - queue_size

        if (datetime.now() - last_time_polled) < self.__min_time_between_polls:
            self.__request_more_records_asynchronously(kafka_consumer=kafka_consumer, last_time_polled=last_time_polled)
            return

        if available_space_in_queue >= 0:
            for kafka_poll_record in kafka_consumer.poll(
                timeout_ms=0,
                number_of_messages_to_poll=self.__max_records_retrieved_per_poll,
            ):
                self.__queue_repository.push((kafka_consumer, kafka_poll_record))

        self.__request_more_records_asynchronously(kafka_consumer=kafka_consumer, last_time_polled=datetime.now())

    def __request_more_records_asynchronously(self, kafka_consumer: KafkaConsumer, last_time_polled: datetime) -> None:
        if kafka_consumer in self.__kafka_consumers_with_poll_requested:
            return

        self.__kafka_consumers_with_poll_requested.add(kafka_consumer)
        self.__polling_threads_pool.submit(self.__poll_kafka_consumer, kafka_consumer, last_time_polled)

    def __consume_kafka_poll_records_from_queue(self) -> None:
        consumer, kafka_poll_record = self.__queue_repository.pop()
        subscriber = self.__subscriber_per_consumer_mapper[consumer]
        consumer.consume(
            kafka_poll_record=kafka_poll_record,
            consumption_callback=lambda kafka_record: self.__consumption_callback(subscriber, kafka_record),
        )

    def __consumption_callback(self, subscriber: Subscriber, message: KafkaConsumerRecord[T]) -> None:
        self.__consume_middleware_chain_resolver.resolve(
            event=message.value, subscriber=subscriber, consume=self.__perform_consume
        )

    def __perform_consume(self, event: T, subscriber: Subscriber) -> None:
        should_retry = True
        while should_retry is True:
            try:
                return subscriber.consume(event)
            except Exception as exc:
                self.__logger.warning(f"Event {event.id} could not be consumed by the subscriber {subscriber.fqn}")

                if self.__should_retry(event, subscriber) is True:
                    self.__register_retry(event, subscriber)
                    continue
                else:
                    if self.__reject_callback:
                        self.__reject_callback.on_reject(event=event, subscribers=[subscriber])

                    if self.__on_fail_strategy == KafkaOnFailStrategy.STOP_ON_FAIL:
                        raise exc

    def __should_retry(self, event: Event, subscriber: Subscriber) -> bool:
        if self.__consume_retrier is None:
            return False

        return self.__consume_retrier.should_retry(event, [subscriber])

    def __register_retry(self, event: Event, subscriber: Subscriber) -> None:
        if self.__consume_retrier is None:
            return None

        return self.__consume_retrier.register_retry(event, [subscriber])

    def stop(self) -> None:
        self.__should_stop.set()

    def __perform_graceful_stop(self) -> None:
        self.__logger.info("Waiting until all polling tasks finish...")
        self.__polling_threads_pool.shutdown()
        self.__logger.info("All polling tasks finished. Stopping kafka consumers...")
        for kafka_consumer in self.__subscriber_per_consumer_mapper.keys():
            kafka_consumer.stop()
        self.__logger.info("All kafka consumers stopped")



from __future__ import annotations

from logging import Logger
from typing import Callable, Optional, Type, cast

from kafka import TopicPartition, OffsetAndMetadata, KafkaConsumer as KafkaPythonLibraryConsumer
from kafka.coordinator.assignors.abstract import AbstractPartitionAssignor

from buz.kafka.domain.exceptions.not_valid_kafka_message_exception import NotValidKafkaMessageException
from buz.kafka.domain.models.consumer_initial_offset_position import ConsumerInitialOffsetPosition
from buz.kafka.domain.models.kafka_connection_config import KafkaConnectionConfig
from buz.kafka.domain.models.kafka_consumer_record import KafkaConsumerRecord
from buz.kafka.infrastructure.deserializers.byte_deserializer import ByteDeserializer
from buz.kafka.infrastructure.kafka_python.kafka_poll_record import KafkaPollRecord
from buz.kafka.infrastructure.kafka_python.translators.consumer_initial_offset_position_translator import (
    ConsumerInitialOffsetPositionTranslator,
)
from buz.kafka.infrastructure.serializers.kafka_header_serializer import KafkaHeaderSerializer


class KafkaPythonMultiThreadedConsumer:
    __DEFAULT_POLL_TIMEOUT_MS = 0
    __DEFAULT_SESSION_TIMEOUT_MS = 1000 * 10
    __DEFAULT_MAX_POLL_INTERVAL = 2147483647

    # https://docs.confluent.io/platform/current/installation/configuration/consumer-configs.html#session-timeout-ms

    def __init__(
        self,
        *,
        consumer_group: str,
        topics: list[str],
        connection_config: KafkaConnectionConfig,
        initial_offset_position: ConsumerInitialOffsetPosition,
        byte_deserializer: ByteDeserializer,
        header_serializer: KafkaHeaderSerializer,
        partition_assignors: tuple[Type[AbstractPartitionAssignor], ...],
        logger: Logger,
        session_timeout_ms: int = __DEFAULT_SESSION_TIMEOUT_MS,
    ) -> None:
        self.__consumer_group = consumer_group
        self.__topics = topics
        self.__initial_offset_position = initial_offset_position
        self.__connection_config = connection_config
        self.__byte_deserializer = byte_deserializer
        self.__header_serializer = header_serializer
        self.__partition_assignors = partition_assignors
        self.__logger = logger
        self.__session_timeout_ms = session_timeout_ms

        self.__consumer = self.__generate_consumer()

    def __generate_consumer(self) -> KafkaPythonLibraryConsumer:
        sasl_mechanism: Optional[str] = None

        if self.__connection_config.credentials.sasl_mechanism is not None:
            sasl_mechanism = self.__connection_config.credentials.sasl_mechanism.value

        consumer = KafkaPythonLibraryConsumer(
            bootstrap_servers=self.__connection_config.bootstrap_servers,
            security_protocol=self.__connection_config.credentials.security_protocol.value,
            sasl_mechanism=sasl_mechanism,
            sasl_plain_username=self.__connection_config.credentials.user,
            sasl_plain_password=self.__connection_config.credentials.password,
            client_id=self.__connection_config.client_id,
            group_id=self.__consumer_group,
            enable_auto_commit=False,
            auto_offset_reset=ConsumerInitialOffsetPositionTranslator.to_kafka_supported_format(
                self.__initial_offset_position
            ),
            session_timeout_ms=self.__session_timeout_ms,
            partition_assignment_strategy=list(self.__partition_assignors),
            max_poll_interval_ms=self.__DEFAULT_MAX_POLL_INTERVAL,
        )

        consumer.subscribe(self.__topics)
        return consumer

    def poll(
        self,
        *,
        timeout_ms: int = __DEFAULT_POLL_TIMEOUT_MS,
        number_of_messages_to_poll: Optional[int] = None,
    ) -> list[KafkaPollRecord]:
        poll_results = self.__consumer.poll(
            timeout_ms=timeout_ms,
            max_records=number_of_messages_to_poll,
        )

        return [
            cast(KafkaPollRecord, consumer_record)
            for consumer_records in poll_results.values()
            for consumer_record in consumer_records
        ]

    def consume(
        self,
        *,
        kafka_poll_record: KafkaPollRecord,
        consumption_callback: Callable[[KafkaConsumerRecord], None],
    ) -> None:
        try:
            if kafka_poll_record.value is None:
                raise NotValidKafkaMessageException("Message is None")

            consumption_callback(
                KafkaConsumerRecord(
                    value=self.__byte_deserializer.deserialize(kafka_poll_record.value),
                    headers=self.__header_serializer.deserialize(kafka_poll_record.headers),
                )
            )
        except NotValidKafkaMessageException:
            # If the message is not valid or if is not we are going to logged it but also we are going to consume it to avoid maintain it in the partition (we currently dont have DLQ or other mechanism)
            self.__logger.error(
                f'The message "{str(kafka_poll_record.value)}" is not valid, it will be consumed but not processed'
            )

        self.__commit_poll_record(kafka_poll_record)
        return

    def __commit_poll_record(self, poll_record: KafkaPollRecord) -> None:
        offset = {
            TopicPartition(topic=poll_record.topic, partition=poll_record.partition): OffsetAndMetadata(
                poll_record.offset + 1, ""
            )
        }

        self.__consumer.commit(offset)

    def stop(self) -> None:
        self.__logger.info(f"Closing connection of consumer with group_id={self.__consumer_group}")
        self.__consumer.close(autocommit=False)



from buz.kafka.domain.models.kafka_connection_config import KafkaConnectionConfig
from buz.kafka.infrastructure.kafka_python.kafka_python_producer import KafkaPythonProducer
from buz.kafka.infrastructure.serializers.byte_serializer import ByteSerializer
from buz.kafka.infrastructure.serializers.implementations.json_byte_serializer import JSONByteSerializer


class KafkaPythonProducerFactory:
    def __init__(
        self,
        kafka_connection_config: KafkaConnectionConfig,
        byte_serializer: ByteSerializer = JSONByteSerializer(),
    ):
        self._kafka_connection_config = kafka_connection_config
        self._byte_serializer = byte_serializer

    def build(self) -> KafkaPythonProducer:
        return KafkaPythonProducer(
            config=self._kafka_connection_config,
            byte_serializer=self._byte_serializer,
        )



from __future__ import annotations

from functools import wraps
from logging import Logger
from typing import Callable, Optional, Type, cast

from kafka import TopicPartition, OffsetAndMetadata, KafkaConsumer as KafkaPythonLibraryConsumer
from kafka.coordinator.assignors.abstract import AbstractPartitionAssignor

from buz.kafka.domain.exceptions.not_valid_kafka_message_exception import NotValidKafkaMessageException
from buz.kafka.domain.models.consumer_initial_offset_position import ConsumerInitialOffsetPosition
from buz.kafka.domain.models.kafka_connection_config import KafkaConnectionConfig
from buz.kafka.domain.models.kafka_consumer_record import KafkaConsumerRecord
from buz.kafka.domain.services.kafka_consumer import (
    KafkaConsumer,
    DEFAULT_NUMBER_OF_MESSAGES_TO_POLL,
)
from buz.kafka.infrastructure.deserializers.byte_deserializer import ByteDeserializer
from buz.kafka.infrastructure.kafka_python.exception.consumer_interrupted_exception import (
    ConsumerInterruptedException,
)
from buz.kafka.infrastructure.kafka_python.kafka_poll_record import KafkaPollRecord
from buz.kafka.infrastructure.kafka_python.translators.consumer_initial_offset_position_translator import (
    ConsumerInitialOffsetPositionTranslator,
)
from buz.kafka.infrastructure.serializers.kafka_header_serializer import KafkaHeaderSerializer


class KafkaPythonConsumer(KafkaConsumer):
    __CONSUMER_POLL_TIMEOUT_MS = 1000
    # https://kafka-python.readthedocs.io/en/master/apidoc/KafkaConsumer.html#kafka.KafkaConsumer.poll
    __SESSION_TIMEOUT_MS = 10000
    # https://docs.confluent.io/platform/current/installation/configuration/consumer-configs.html#session-timeout-ms

    def __init__(
        self,
        *,
        consumer_group: str,
        topics: list[str],
        connection_config: KafkaConnectionConfig,
        initial_offset_position: ConsumerInitialOffsetPosition,
        byte_deserializer: ByteDeserializer,
        header_serializer: KafkaHeaderSerializer,
        partition_assignors: tuple[Type[AbstractPartitionAssignor], ...],
        logger: Logger,
    ) -> None:
        self.__consumer_group = consumer_group
        self.__topics = topics
        self.__initial_offset_position = initial_offset_position
        self.__connection_config = connection_config
        self.__byte_deserializer = byte_deserializer
        self.__header_serializer = header_serializer
        self.__partition_assignors = partition_assignors
        self.__logger = logger

        self.__consumer = self.__generate_consumer()
        self.__offsets_metadata: dict[TopicPartition, OffsetAndMetadata] = {}
        self.__gracefully_stop = False

    def __generate_consumer(self) -> KafkaPythonLibraryConsumer:
        sasl_mechanism: Optional[str] = None

        if self.__connection_config.credentials.sasl_mechanism is not None:
            sasl_mechanism = self.__connection_config.credentials.sasl_mechanism.value

        consumer = KafkaPythonLibraryConsumer(
            bootstrap_servers=self.__connection_config.bootstrap_servers,
            security_protocol=self.__connection_config.credentials.security_protocol.value,
            sasl_mechanism=sasl_mechanism,
            sasl_plain_username=self.__connection_config.credentials.user,
            sasl_plain_password=self.__connection_config.credentials.password,
            client_id=self.__connection_config.client_id,
            group_id=self.__consumer_group,
            enable_auto_commit=False,
            auto_offset_reset=ConsumerInitialOffsetPositionTranslator.to_kafka_supported_format(
                self.__initial_offset_position
            ),
            session_timeout_ms=self.__SESSION_TIMEOUT_MS,
            partition_assignment_strategy=list(self.__partition_assignors),
        )

        consumer.subscribe(self.__topics)
        return consumer

    def __gracefully_handle_consumer_interruption(func: Callable) -> Callable:  # type: ignore[misc]
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            if self.__gracefully_stop is True:
                self.__close_consumer()
                return

            try:
                return func(self, *args, **kwargs)
            except ConsumerInterruptedException:
                self.__close_consumer()
                return
            except Exception:
                self.__close_consumer()
                raise

        return wrapper

    @__gracefully_handle_consumer_interruption  # noqa
    def consume(
        self,
        *,
        consumption_callback: Callable[[KafkaConsumerRecord], None],
        number_of_messages_to_poll: int = DEFAULT_NUMBER_OF_MESSAGES_TO_POLL,
    ) -> None:
        self.__offsets_metadata.clear()
        poll_results = self.__consumer.poll(
            timeout_ms=self.__CONSUMER_POLL_TIMEOUT_MS,
            max_records=number_of_messages_to_poll,
        )

        for topic_partition, consumer_records in poll_results.items():
            for consumer_record in consumer_records:
                kafka_poll_record = cast(KafkaPollRecord, consumer_record)

                try:
                    if kafka_poll_record.value is None:
                        raise NotValidKafkaMessageException("The kafka poll record value is None")

                    consumption_callback(
                        KafkaConsumerRecord(
                            value=self.__byte_deserializer.deserialize(kafka_poll_record.value),
                            headers=self.__header_serializer.deserialize(kafka_poll_record.headers),
                        )
                    )
                except NotValidKafkaMessageException:
                    self.__logger.error(
                        f'The message "{str(kafka_poll_record.value)}" is not valid, it will be consumed but not processed'
                    )

                self.__mark_record_as_consumed(kafka_poll_record)

                if self.__gracefully_stop is True:
                    raise ConsumerInterruptedException()

        self.__consumer.commit()

    def __mark_record_as_consumed(self, consumer_record: KafkaPollRecord) -> None:
        """
        the committed offset should be
        the next message your application should consume, i.e.: last_offset + 1.
        """
        next_offset_to_be_consumed = consumer_record.offset + 1
        self.__offsets_metadata[
            TopicPartition(topic=consumer_record.topic, partition=consumer_record.partition)
        ] = OffsetAndMetadata(next_offset_to_be_consumed, "")

    def __close_consumer(self) -> None:
        self.__consumer.commit(offsets=self.__offsets_metadata)

        self.__logger.info(f"Closing connection of consumer with group_id={self.__consumer_group}")
        self.__consumer.close(autocommit=False)

    def request_stop(self) -> None:
        self.__gracefully_stop = True
